<?php
declare(strict_types=1);

namespace Flat101\EnhancedEcommerce\Controller\Click;

use Flat101\EnhancedEcommerce\Factory\ClickImpressionHandlerFactory;
use Flat101\EnhancedEcommerce\Model\DataLayer\Impressions\ClickImpressionView as ClickImpressionViewUa;
use Flat101\EnhancedEcommerce\Model\DataLayerGa4\Impressions\ClickImpressionView as ClickImpressionViewGa4;
use Magento\Catalog\Api\ProductRepositoryInterface;
use Magento\Framework\App\ActionInterface;
use Magento\Framework\App\Response\RedirectInterface;
use Magento\Framework\Serialize\SerializerInterface;
use Psr\Log\LoggerInterface;
use Magento\Framework\App\RequestInterface;
use Magento\Framework\App\ResponseInterface;

class View implements ActionInterface
{
    /** @var RedirectInterface */
    protected $redirect;

    /** @var ClickImpressionHandlerFactory */
    protected $clickImpressionHandlerFactory;

    /** @var ProductRepositoryInterface */
    protected $productRepository;

    /** @var ClickImpressionViewUa */
    protected $clickImpressionView;

    /** @var ClickImpressionViewGa4 */
    protected $clickImpressionViewGa4;

    /** @var SerializerInterface */
    protected $serializer;

    /** @var LoggerInterface */
    protected $logger;

    /** @var RequestInterface */
    protected $request;

    /** @var ResponseInterface */
    protected $response;

    public function __construct(
        RedirectInterface $redirect,
        ClickImpressionHandlerFactory $clickImpressionHandlerFactory,
        ProductRepositoryInterface $productRepository,
        ClickImpressionViewUa $clickImpressionView,
        ClickImpressionViewGa4 $clickImpressionViewGa4,
        SerializerInterface $serializer,
        LoggerInterface $logger,
        RequestInterface $request,
        ResponseInterface $response
    ) {
        $this->redirect = $redirect;
        $this->clickImpressionHandlerFactory = $clickImpressionHandlerFactory;
        $this->productRepository = $productRepository;
        $this->clickImpressionView = $clickImpressionView;
        $this->clickImpressionViewGa4 = $clickImpressionViewGa4;
        $this->serializer = $serializer;
        $this->logger = $logger;
        $this->request = $request;
        $this->response = $response;
    }

    /**
     * {@inheritdoc}
     */
    public function execute()
    {
        if ($this->request->isAjax()) {
            $currentUrl = $this->request->getParam('currentUrl');
            $refererUrl = $this->request->getParam('refererUrl');
            $isDisabledUaDatalayer = $this->request->getParam('isDisabledUaDatalayer') === 'true';

            if (null === $refererUrl || $refererUrl === $currentUrl) {
                return $this->response->representJson('');
            }

            $productId = (int) $this->request->getParam('id');
            $clickDataGa4 = $this->serializer->unserialize($this->request->getParam('clickDataGa4'));
            $clickDataUa = $this->serializer->unserialize($this->request->getParam('clickDataUa'));
            $clickDataMerged = [
                'GA4' => $clickDataGa4,
                'UA'  => $clickDataUa
            ];
            if ($productId) {
                try {
                    $product = $this->productRepository->getById($productId);

                    $handlerImpressionClick = $this->clickImpressionHandlerFactory->create($refererUrl);
                    $handlerImpressionClick->handle($product, $clickDataMerged);

                    $layers = [];
                    $layers['GA4'] = $this->clickImpressionViewGa4->getLayer();
                    if (!$isDisabledUaDatalayer) {
                        $layers['UA'] = $this->clickImpressionView->getLayer();
                    }

                    return $this->response->representJson(
                        $this->serializer->serialize($layers)
                    );
                } catch (\Exception $e) {
                    $this->logger->critical($e->getMessage());
                }
            }
        }

        return $this->response->representJson('');
    }
}
