<?php declare(strict_types=1);

namespace Flat101\EnhancedEcommerce\Helper;

use Flat101\EnhancedEcommerce\Helper\Config;
use Magento\Framework\App\Helper\AbstractHelper;
use Magento\Store\Model\StoreManagerInterface;
use Magento\Framework\HTTP\Client\Curl;
use Magento\Framework\HTTP\Header;
use Psr\Log\LoggerInterface;

class MeasurementProtocol extends AbstractHelper
{
    /**
     * @var StoreManagerInterface
     */
    protected $storeManager;

    /**
     * @var Curl
     */
    protected $curl;

    /**
     * @var Header
     */
    protected $http;

    /** @var Config */
    protected $configHelper;

    /** @var LoggerInterface\ */
    protected $logger;


    /**
     * @param StoreManagerInterface $storeManager
     * @param Curl $curl
     * @param Header $http
     * @param Config $configHelper
     * @param LoggerInterface $logger
     */
    public function __construct(
        StoreManagerInterface $storeManager,
        Curl $curl,
        Header $http,
        Config $configHelper,
        LoggerInterface $logger
    ) {
        $this->storeManager = $storeManager;
        $this->curl = $curl;
        $this->http = $http;
        $this->configHelper = $configHelper;
        $this->logger = $logger;
    }

    public function sendDataToGa4($payload, $storeId, $cid)
    {
        $url = $this->getUrl($storeId);
        $cid = $this->cleanCid($cid);

        $data = [
            'client_id' => $cid,
            'events'    => $payload
        ];

        $this->sendData(json_encode($data), $url);
    }

    private function getUrl($storeId)
    {
        $url = '';
        if ($this->configHelper->isGa4DebugModeEnabled($storeId)) {
            $url = $this->configHelper->getGa4DebugUrl($storeId);
        } else {
            $url = $this->configHelper->getGa4Url($storeId);
        }

        $api_secret = $this->configHelper->getSecretId($storeId);
        $measurement_id = $this->configHelper->getMeasurementId($storeId);
        return $url . '?measurement_id=' . $measurement_id . '&api_secret=' . $api_secret;

    }

    /**
     * @throws \Exception
     */
    private function sendData($data, $url)
    {
        $userAgent = $this->http->getHttpUserAgent();
        if (!$userAgent) { // User agent is a required option
            $userAgent = 'Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.1.13) Gecko/20080311
Firefox/2.0.0.13';
        }

        $curlOptions = array(
            CURLOPT_SSL_VERIFYPEER => 0,
            CURLOPT_TIMEOUT => 10,
            CURLOPT_USERAGENT => $userAgent,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1
        );
        try {
            $this->logger->info('GA4 - URL to send info: ' . $url);
            $this->logger->info('GA4 - Payload sent:' . print_r($data, true));

            $this->curl->addHeader("Content-Type", "application/json");
            $this->curl->setOptions($curlOptions);
            $this->curl->post($url, $data);

            $response = $this->curl->getBody();
            $this->logger->info($response);
            if ($this->curl->getStatus() != 200) {
                return false;
            }
        } catch (\Exception $e) {
            $this->logger->info('GA4 - Error calling to GA4: ' . $e->getMessage());
            throw new \Exception('Error sending data to GA4: ' . $e->getMessage());
        }
        return true;
    }

    public function cleanCid($cid)
    {
        if (substr($cid, 0, 2 ) === "GA") {
            $cid = substr($cid, 6);
        }

        return $cid;
    }
}
