<?php
declare(strict_types=1);

namespace Flat101\EnhancedEcommerce\Model\Handler;

use Flat101\EnhancedEcommerce\Model\DataLayer\Generator\Impression;
use Flat101\EnhancedEcommerce\Model\DataLayerGa4\Generator\Impression as ImpressionGa4;
use Flat101\Base\Service\GetCurrentCategoryService As CategoryRegistry;
use Flat101\EnhancedEcommerce\Model\Session;
use Flat101\EnhancedEcommerce\Helper\Config;
use Magento\Catalog\Model\ResourceModel\Product\Collection;
use Magento\Catalog\Block\Product\ProductList\Toolbar;

class CatalogImpressionHandler implements ImpressionHandlerInterface
{
    /** @var Session */
    private $session;

    /** @var Impression */
    private $impressionGenerator;

    /** @var ImpressionGa4 */
    private $impressionGeneratorGa4;

    /** @var Toolbar */
    protected $toolbar;

    /** @var CategoryRegistry */
    protected $categoryRegistry;

    /** @var Config */
    protected $configHelper;

    public function __construct(
        Session $session,
        Impression $impressionGenerator,
        ImpressionGa4 $impressionGeneratorGa4,
        Toolbar $toolbar,
        CategoryRegistry $categoryRegistry,
        Config $configHelper
    ) {
        $this->session = $session;
        $this->impressionGenerator = $impressionGenerator;
        $this->impressionGeneratorGa4 = $impressionGeneratorGa4;
        $this->toolbar = $toolbar;
        $this->categoryRegistry = $categoryRegistry;
        $this->configHelper = $configHelper;
    }

    public function handle(Collection $collection): void
    {
        $this->session->getImpressionCatalogProducts(true,true);
        $this->session->getImpressionCatalogProducts(false,true);

        $listName = $this->configHelper->getListLabelCatalogImpressions($this->categoryRegistry->getCategory()->getName());
        $pageSize = $this->toolbar->getLimit();
        $pageNumber = $this->toolbar->getCurrentPage();
        $position = ($pageNumber - 1) * $pageSize;

        /** @var \Magento\Catalog\Model\Product $product */
        foreach ($collection as $product) {
            $position++;
            // Set session data for GA4 datalayer
            $this->session->setImpressionCatalogProducts(
                $this->impressionGeneratorGa4->generate($product, $listName,
                    ['position' => $position]),
                false
            );
            // Set session data for UA datalayer if it's turned on
            if (!$this->configHelper->getIsDisabledUaDatalayer()) {
                $this->session->setImpressionCatalogProducts(
                    $this->impressionGenerator->generate($product, $listName,
                        ['position' => $position]),
                    true
                );
            }
        }
    }
}
